# Copyright 2012, 2013 Marco Ceppi, Canonical Ltd.  This software is
# licensed under the GNU Affero General Public License version 3 (see
# the file LICENSE).

"""
Juju Core Review

 Contributed by Juan Negronil


 Changes
 - convert to job.
 - use dateutil date parsing instead of string comparsion.
 - merge proposals show target with series/branch.

"""

from launchpadlib.launchpad import Launchpad

import pymongo
import logging
import time

from charmworld.utils import (
    configure_logging,
    get_ini,
)
from config import MONGO_URL
from utils import lock
from utils import LockHeld

log = None


def update_review_queue(db):
    st = time.time()
    lp = Launchpad.login_anonymously('charm-tools',
                                     'production',
                                     version='devel')

    # Entity roots for items of interests
    #charm = lp.distributions['charms']
    #charmers = lp.people['charmers']
    #charm_contributors = lp.people['charm-contributors']

    log.debug("lp login time %0.2f", time.time() - st)

    st = time.time()
    #bugs = charm.searchTasks(
    #    tags=['new-formula', 'new-charm'],
    #    tags_combinator="Any",
    #    status=[
    #        'New', 'Confirmed', 'Triaged', 'In Progress', 'Fix Committed'])
    #charmers_bugs = charmers.searchTasks(
    #    status=[
    #        'New', 'Confirmed', 'Triaged', 'In Progress', 'Fix Committed'])
    #proposals = charmers.getRequestedReviews(status="Needs review")
    #contributor_proposals = charm_contributors.getRequestedReviews(
    #    status="Needs review")

    juju = lp.projects['juju']
    branch_proposals = [
        mp for mp in juju.getMergeProposals()
        if mp.target_branch_link.endswith("~juju/juju/trunk")]
    branch_proposals = [
        mp for mp in branch_proposals if not mp.queue_status in (
            'Work in progress',)]
    log.debug("lp query time %0.2f", time.time() - st)

    r_queue = list()

    for proposal in branch_proposals:
        parts = proposal.target_branch_link.rsplit('/', 3)
        if parts[-1] == "trunk":
            parts.pop(-1)

        target = "/".join(parts[2:])
        origin = "/".join(proposal.source_branch_link.rsplit('/', 3)[1:])

        num_comments = proposal.all_comments.total_size
        if num_comments != 0:
            last_modified = \
                proposal.all_comments[num_comments - 1].date_created
        else:
            last_modified = proposal.date_created

        entry = {
            'date_modified': last_modified,
            'date_created': proposal.date_created,
            'summary': "Merge for %s from %s" % (target, origin),
            'item': proposal.web_link,
            'status': proposal.queue_status,
        }
        r_queue.append(entry)

    # Drop old collection
    log.info("Dropping old review queue (%d items)",
             db['core_review_queue'].count())
    db.drop_collection('core_review_queue')

    # Bulk insert
    log.info("Updating review queue (%d items)" % len(r_queue))
    db['core_review_queue'].insert(r_queue)


def main():
    global log
    configure_logging()
    log = logging.getLogger("core.review")
    c = pymongo.Connection(MONGO_URL, safe=True)
    db = c['juju']
    settings = get_ini()
    try:
        with lock('ingest-core-review',
                  int(settings['script_lease_time']) * 60, db, log):
            update_review_queue(db)
    except LockHeld, error:
        log.warn(str(error))

if __name__ == '__main__':
    main()
