# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import logging

from charmworld.models import (
    Charm,
    construct_charm_id,
    FeaturedSource,
    QADataSource,
)
from charmworld.qa_questions import iter_categories


def ensure_category_questions(db):
    db.qa.drop()
    for category_dict in iter_categories():
        db.qa.insert(category_dict)


def upgrade_qa_12(db, charm_source):
    """Change the representation of qa answers."""
    qa_data_source = QADataSource.from_db(db)
    for charm_data in charm_source.collection.find({'qa': {'$exists': True}}):
        qa_data_source.save_qa_data(Charm(charm_data), charm_data['qa'])
        del charm_data['qa']
        charm_source.save(charm_data)


def upgrade_migrate_featured_13(db, index_client):
    featured_data = index_client.api_search(filters={'is_featured': [True]})
    featured = FeaturedSource(db.featured)
    for charm_data in featured_data:
        if charm_data.get('is_featured'):
            featured.set_featured(charm_data, 'charm')


def exodus_update(source, target, current_version):
    log = logging.getLogger("charm.migration")
    db = source.collection.database
    for charm in source.collection.find():
        charm['_id'] = construct_charm_id(charm)
        charm.pop('doctype', None)
        try:
            target.save(charm)
        except Exception as e:
            log.exception(str(e))
    # QA questions should always be in the migration list, so that dev
    # instances have them.
    ensure_category_questions(db)
    if current_version < 12:
        upgrade_qa_12(db, target)
    if current_version < 13:
        upgrade_migrate_featured_13(db, target.index_client)
