/* Copyright 2012 Canonical Ltd.  This software is licensed under the
 * GNU Affero General Public License version 3 (see the file LICENSE).
 */

Ext.require("Ext.data.Store");
Ext.require("Ext.data.Model");
Ext.require('Ext.chart.*');
Ext.require('Ext.layout.container.Fit');

Ext.define('CharmActivity', {
    extend: 'Ext.data.Model',
    proxy: {
        type: 'ajax',
        url : '/reports/charm-activity.json',
        reader: {
	    type: 'json',
	    root: 'series'}
    },
    autoLoad: true,
    fields: ['timespan', 'value']
});


Ext.define('CharmGrowth', {
    extend: 'Ext.data.Model',
    proxy: {
        type: 'ajax',
        url : '/reports/charm-count.json',
        reader: {
	    type: 'json',
	    root: 'series'}
    },
    autoLoad: true,
    fields: ['timespan', 'ppa', 'distro']
});


Ext.define('ContributorGrowth', {
    extend: 'Ext.data.Model',
    proxy: {
        type: 'ajax',
        url : '/reports/charm-contributors.json',
	reader: {
	    type: 'json',
	    root: 'series'}
    },
    autoLoad: true,
    fields: ['timespan', 'community', 'canonical', 'juju']
});


Ext.onReady(function(){
    var charm_activity = Ext.create('Ext.data.Store', {
	model: 'CharmActivity'
    });

    var charm_growth = Ext.create('Ext.data.Store', {
	model: 'CharmGrowth'
    });

    var contributor_growth = Ext.create('Ext.data.Store', {
	model: 'ContributorGrowth'
    });

    // Load the data stores.
    charm_activity.load();
    charm_growth.load();
    contributor_growth.load();

    Ext.create('Ext.chart.Chart', {
	width: 600,
	height: 400,
	insetPadding: 30,
	store: charm_activity,
	animate: true,
	shadow: true,
	renderTo: Ext.get('charm-activity'),
	// Set a legend
	axes: [{
            type: 'Numeric',
            position: 'left',
            fields: ['value'],
            title: 'Commits',
	    grid: true,
            //minimum: 0,
            //one minor tick between two major ticks
            minorTickSteps: 5,
	}, {
            type: 'Category',
            position: 'bottom',
            fields: ['timespan'],
            title: 'Week',
	    label: {
		rotate: { 
		    degrees: 315
		}}
	}],

	series: [{
            type: 'line',
            xField: 'timespan',
            yField: 'value',
            tips: {
                trackMouse: true,
                width: 80,
                height: 40,
                renderer: function(storeItem, item) {
                    this.setTitle(
			storeItem.get('value') + ' Commits <br/> ' + storeItem.get('timespan'));
                }},
	}]
    }); 

    Ext.create('Ext.chart.Chart', {
	width: 600,
	height: 400,
	store: charm_growth,
	legend: {
	    position: 'bottom'
	},
	renderTo: Ext.get('charm-growth'),
	axes: [{
            type: 'Numeric',
            position: 'left',
            fields: ['ppa', 'distro'],
            title: 'Charms',
	    grid: true,
	}, {
            type: 'Category',
            position: 'bottom',
            fields: ['timespan'],
            title: 'Week',

	    label: {
		rotate: { 
		    degrees: 315
		}}
	}],

	series: [{
            type: 'area',
            xField: 'timespan',
            yField: ['distro', 'ppa'],

            tips: {
                trackMouse: true,
                width: 150,
                height: 25,
                renderer: function(storeItem, item) {
                    this.setTitle(
			item.storeField + ' - ' + storeItem.get(item.storeField) + ' Charms: ' + storeItem.get('timespan'));
                }},
	}]
    }); 

    Ext.create('Ext.chart.Chart', {
	width: 600,
	height: 400,
	store: contributor_growth,
	animate: true,
	shadow: true,
	legend: {
	    position: 'bottom'
	},
	renderTo: Ext.get('contributor-growth'),
	axes: [{
            type: 'Numeric',
            position: 'left',
            fields: ['juju', 'community', 'canonical'],
            title: 'Contributors',
	    grid: true,
	}, {
            type: 'Category',
            position: 'bottom',
            fields: ['timespan'],
            title: 'Week',

	    label: {
		rotate: { 
		    degrees: 315
		}}
	}],

	series: [{
            type: 'area',
            xField: 'timespan',
            yField: ['juju', 'community', 'canonical'],
            tips: {
                trackMouse: true,
                width: 130,
                height: 25,
                renderer: function(storeItem, item) {
                    this.setTitle(
			storeItem.get(item.storeField) + ' ' + item.storeField + ' - ' + storeItem.get('timespan'));
                }},
	}]
    }); 

})

