# Copyright 2009-2013 - Curtis Hovey <sinzui.is at verizon.net>
# Copyright 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from cStringIO import StringIO
from htmlentitydefs import entitydefs
import re
from xml.etree import ElementTree


xml_decl_pattern = re.compile(r'<\?xml .*?\?>')


xhtml_doctype = (
    '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" '
    '"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">')


def parse(text):
    """Parse an HTML document using ElementTree."""
    # Reconcile the text and Expat checker text requriements.
    if text == '':
        return
    parser = ElementTree.XMLParser()
    parser.entity.update(entitydefs)
    # The expat parser seems to be assuming ascii even when
    # XMLParser(encoding='utf-8') is used above.
    text = text.encode('utf-8').decode('ascii', 'ignore')
    if text.find('<!DOCTYPE') == -1:
        # Expat requires a doctype to honour parser.entity.
        match = xml_decl_pattern.search(text)
        if match is None:
            text = xhtml_doctype + '\n' + text
        else:
            start, end = match.span(0)
            text = text[:start] + xhtml_doctype + '\n' + text[end:]
    elif text.find('<!DOCTYPE html>') != -1:
        text = text.replace('<!DOCTYPE html>', xhtml_doctype)
    return ElementTree.parse(StringIO(text), parser)


def find_by_id(tree_element, element_id):
    """Find an element by its id, using another element from the document."""
    return tree_element.getroot().find('.//*[@id="%s"]' % element_id)


def find_answer_field(tree, question_id, value):
    """Find a field for a particular answer to a question.

    Mainly useful for finding radio buttons or checkboxes.
    :param question_id: The ID of the question to find the answer for.
    :param value: The value of the answer for the field.
    """
    question = tree.find(
        './/input[@value="%s:rename"]/..' % question_id)
    answer, = question.findall('.//input[@value="%s"]' % value)
    return answer
