# Copyright 2013 Canonical Ltd.  This software is licensed under the GNU
# Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type


import json

from httmock import HTTMock

from charmworld.lp import (
    API,
    get_branch_info,
    get_branch_tips,
)
from charmworld.testing import (
    response404,
    TestCase,
)


class TestLP(TestCase):

    def mocked_response(self, url, json_result):

        def available_mock(url_, request):
            self.assertEqual(url, request.url)
            return json.dumps(json_result)

        return HTTMock(available_mock)

    def test_get_branch_tips(self):
        charm_data = [[u'~charmers/charms/precise/someproject/trunk',
                       u'ja@appflower.com-20120329093714-s2m9e28dwotmijqc',
                       []]]

        def available_mock(url, request):
            self.assertEqual(
                request.url,
                'https://api.launchpad.net/devel/charm?ws.op=getBranchTips')
            return json.dumps(charm_data)

        mock = self.mocked_response(
            'https://api.launchpad.net/devel/charm?ws.op=getBranchTips',
            charm_data)
        with mock:
            charms = get_branch_tips()
        self.assertEqual(charm_data, charms)

    def test_get_branch_info(self):
        result = {'date_created': '2012-03-06T17:59:55.818912+00:00'}
        mock = self.mocked_response(API + '/~abentley/foo/bar',
                                    result)
        with mock:
            branch_info = get_branch_info('~abentley/foo/bar')
        self.assertEqual(result, branch_info)

    def test_get_branch_info_404(self):
        with HTTMock(response404):
            self.assertIs(None, get_branch_info('~abentley/foo/bar'))
