# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from datetime import datetime
import pymongo
from webhelpers import feedgenerator
from webob import Response

from charmworld import cached_view_config
from charmworld.models import Charm
from charmworld.views.helpers import format_change
from charmworld.views.helpers import found
from charmworld.views.helpers import name_filter


def add_change_item(feed, charm, change):
    committer = name_filter(change['committer'])
    diff_link = u"https://bazaar.launchpad.net/%s/revision/%s" % (
        charm.branch_spec, change["revno"])
    feed.add_item(
        title=charm.name,
        link=diff_link,
        description=change['message'].strip().replace("\n", "<br/>"),
        author_name=committer,
        pub_date=feedgenerator.rfc3339_date(
            datetime.utcfromtimestamp(change['created'])))


def _charm_changes_feed(charm, request):
    feed = feedgenerator.Atom1Feed(
        title=u"Charm browser changes",
        description="Recently changed charms",
        link=request.url,
        language=u"en"
    )
    for change in charm.changes:
        add_change_item(feed, charm, change)
    return feed


def _recently_changed(request):
    projection = {
        "last_change": 1,
        "name": 1,
        "short_url": 1,
        "summary": 1,
        "branch_spec": 1,
        'store_url': 1,
    }
    sort = [("last_change.created", pymongo.DESCENDING)]
    charms = [
        Charm(data) for data in request.db.charms.find(
            {}, projection, sort=sort, limit=50)]
    return {"charms": charms, 'format_change': format_change}


@cached_view_config(
    route_name="recently-changed",
    renderer="charmworld:templates/recent.pt",
    http_cache_modifier=.5)
def recently_changed(request):
    return _recently_changed(request)


@cached_view_config(
    route_name="recently-changed-feed")
def recently_changed_feed(request):
    feed = feedgenerator.Atom1Feed(
        title=u"Charm browser changes",
        description="Recently changed charms",
        link=request.url,
        language=u"en"
    )
    for charm in _recently_changed(request)['charms']:
        add_change_item(feed, charm, charm.last_change)
    return Response(feed.writeString("utf8"),
                    content_type=u"application/atom+xml")


@cached_view_config(
    route_name="personal-charm-changes")
def personal_charm_changes(request):
    charm = found(request.db.charms.find_one(
        {"name": request.matchdict['charm'],
         "owner": request.matchdict['owner'],
         "series": request.matchdict['series']}))
    feed = _charm_changes_feed(Charm(charm), request)
    return Response(feed.writeString("utf8"),
                    content_type=u"application/atom+xml")


@cached_view_config(
    route_name="charm-changes")
def charm_changes(request):
    charm = found(request.db.charms.find_one(
        {"name": request.matchdict['charm'],
         "promulgated": True,
         "series": request.matchdict['series']}))
    feed = _charm_changes_feed(Charm(charm), request)
    return Response(feed.writeString("utf8"),
                    content_type=u"application/atom+xml")
