# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from collections import defaultdict

from pyramid.view import view_config

from charmworld import cached_view_config
from charmworld.health import (
    check_api2,
    check_api3,
    check_elasticsearch,
    check_mongodb,
)


# Basic app views
@cached_view_config(
    route_name="home",
    renderer="charmworld:templates/index.pt")
def home(request):
    return {}


@view_config(
    route_name="charmers",
    renderer="charmworld:templates/charmers.pt")
def charmers(request):
    # We want to collect which users have contributes on which charms.
    # This however reports charm owners ordered decending by ownership.
    # The largest owner is not included because we think this is ~charmers.
    results = request.db.charms.find(
        {}, {"name": 1, 'owner': 1, 'short_url': 1})

    owner_map = defaultdict(int)
    for i in results:
        owner_map[i['owner']] += 1
    owners = sorted(
        owner_map.items(), key=lambda i: (i[1], i[0]), reverse=True)
    owners.pop(0)
    return {"owners": owners}


@view_config(
    route_name="heartbeat",
    http_cache=(0, {'public': True}),
    renderer="charmworld:templates/heartbeat.pt")
def heartbeat(request):
    """Return a dict summarising the operational state of charmworld."""
    checks = []
    checks.append(check_mongodb(request))
    checks.append(check_elasticsearch(request))
    checks.append(check_api2(request))
    checks.append(check_api3(request))
    return {'checks': checks}
