# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import json
from pyramid.httpexceptions import HTTPFound
from pyramid.renderers import render_to_response
from pyramid.view import view_config
from webob import Response

from charmworld.search import (
    CHARM,
    IndexNotReady,
    SearchServiceNotAvailable,
)

from charmworld.views.api import json_response
from charmworld.views.helpers import result_sorter


def do_search(request):
    return request.index_client.search(request.params["search_text"])


@view_config(
    route_name="search",
    renderer="charmworld:templates/search.pt")
def search(request):
    if not "search_text" in request.params:
        url = request.route_url("home")
        return HTTPFound(location=url)
    try:
        search = do_search(request)
    except (IndexNotReady, SearchServiceNotAvailable):
        response = render_to_response('../templates/search-not-ready.pt', {},
                                      request=request)
        response.status_code = 503
        return response
    return dict((key, value) for key, value in search.items()
                if key != 'matches')


@view_config(
    route_name="search-json")
def search_json(request):
    try:
        data = do_search(request)
    except (IndexNotReady, SearchServiceNotAvailable):
        return json_response(503, {'type': 'search_not_ready'})
    data['charm_total'] = data.pop('result_total')
    results = data['results'][CHARM]
    charms = []
    for r in results:
        charm = r['data']
        if charm.store_url == '':
            continue

        charms.append({
            'store_url': charm.store_url,
            'name': charm.name,
            'summary': charm.summary,
            'series': charm.series,
            'owner': charm.owner,
            'data_url': '%s/json' % (
                # request.host_url,
                charm.short_url),
            'relevance': r['weight'],
        })

        if 'subordinate' in data:
            charms[-1]['subordinate'] = True

    charms.sort(result_sorter)

    data.update({"results": charms,
                 "results_size": len(charms)})

    return Response(
        json.dumps(data, indent=2),
        headerlist=[
            ("Access-Control-Allow-Origin", "*"),
            ("Access-Control-Allow-Headers", "X-Requested-With")],
        content_type=u"application/json")
