# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import pymongo
from pyramid.view import view_config

from charmworld import utils
from charmworld.models import (
    CharmSource
)
from charmworld.views.helpers import format_proof


# 3m
REPORT_CACHE = 180


@view_config(
    route_name="tools-core-review",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/review.pt")
def core_review(request):
    # This view's data in a separate cron job than ingest
    queue = list(
        request.db.core_review_queue.find(
            sort=[('date_modified', pymongo.ASCENDING)]))
    return {'queue': queue, 'age_format': utils.pretty_timedelta}


@view_config(
    route_name="tools-review",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/review.pt")
def charm_review(request):
    # This view's data in a separate cron job than ingest
    queue = list(
        request.db.review_queue.find(
            sort=[('date_modified', pymongo.ASCENDING)]))
    return {'queue': queue, 'age_format': utils.pretty_timedelta}


@view_config(
    route_name="tools-store-missing",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/store-missing.pt")
def charm_store_missing(request):
    queue = list(CharmSource.from_request(request).find_mia_charms())
    return {'queue': queue, 'age_format': utils.pretty_timedelta}


@view_config(
    route_name="tools-proof-errors",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/charm-proof-errors.pt")
def proof_errors(request):
    sub_only = bool(request.GET.get('sub'))
    source = CharmSource.from_request(request)
    charms = list(source.find_proof_error_charms(sub_only))

    return {
        'charms': charms,
        'format_proof': format_proof,
        'sub_only': sub_only,
    }
